#include <sys/types.h>

#define DEFAULT_MEM_DEV "/dev/mem"

/* Use memory alignment workaround or not */
#ifdef __ia64__
#define ALIGNMENT_WORKAROUND
#endif


typedef unsigned char u8;
typedef unsigned short u16;
typedef signed short i16;
typedef unsigned int u32;


#ifdef BIGENDIAN
typedef struct {
    u32 h;
    u32 l;
} u64;
#else
typedef struct {
    u32 l;
    u32 h;
} u64;
#endif

#if defined(ALIGNMENT_WORKAROUND) || defined(BIGENDIAN)
static __inline u64 U64(u32 low, u32 high)
{
    u64 self;

    self.l = low;
    self.h = high;

    return self;
}
#endif

/*
 * Per SMBIOS v2.8.0 and later, all structures assume a little-endian
 * ordering convention.
 */
#if defined(ALIGNMENT_WORKAROUND) || defined(BIGENDIAN)
#define WORD(x) (u16)((x)[0] + ((x)[1] << 8))
#define DWORD(x) (u32)((x)[0] + ((x)[1] << 8) + ((x)[2] << 16) + ((x)[3] << 24))
#define QWORD(x) (U64(DWORD(x), DWORD(x + 4)))
#else /* ALIGNMENT_WORKAROUND || BIGENDIAN */
#define WORD(x) (u16)(*(const u16 *)(x))
#define DWORD(x) (u32)(*(const u32 *)(x))
#define QWORD(x) (*(const u64 *)(x))
#endif /* ALIGNMENT_WORKAROUND || BIGENDIAN */


#define ARRAY_SIZE(x) (sizeof(x)/sizeof((x)[0]))

int checksum(const u8 *buf, size_t len);
void *read_file(off_t base, size_t *len, const char *filename);
void *mem_chunk(off_t base, size_t len, const char *devmem);
int write_dump(size_t base, size_t len, const void *data, const char *dumpfile, int add);
u64 u64_range(u64 start, u64 end);

struct string_keyword
{
    const char *keyword;
    u8 type;
    u8 offset;
};

struct opt
{
    const char *devmem;
    unsigned int flags;
    u8 *type;
    const struct string_keyword *string;
    char *dumpfile;
    u32 handle;
};
extern struct opt opt;

#define FLAG_VERSION            (1 << 0)
#define FLAG_HELP               (1 << 1)
#define FLAG_DUMP               (1 << 2)
#define FLAG_QUIET              (1 << 3)
#define FLAG_DUMP_BIN           (1 << 4)
#define FLAG_FROM_DUMP          (1 << 5)
#define FLAG_NO_SYSFS           (1 << 6)

int parse_command_line(const char *str);

struct dmi_header
{
    u8 type;
    u8 length;
    u16 handle;
    u8 *data;
};

char *dmi_get(const char *arg);
void deblank(char string[]);
const char *dmi_string(const struct dmi_header *dm, u8 s);
char *dmi_oem(const struct dmi_header *dm, u8 s);
const char *dmi_chassis_type(u8 code);
const char *dmi_processor_family(const struct dmi_header *h, u16 ver);
char *dmi_modalias(void);